//
//   y = c0 + c1 * x + c2 * x**2
//
#include <stdio.h>
#include <gsl/gsl_multifit.h>

int main (int argc, char **argv)
{
    int i, n;
    double xi, yi, ei, chisq;
    gsl_matrix *X, *cov;
    gsl_vector *y, *w, *c;

    if (argc != 2){  fprintf (stderr,"usage: fit n < data\n");  exit (-1); }

    n = atoi (argv[1]);             // Number of data

    X   = gsl_matrix_alloc(n, 3);   // X = (1, x, x^2 )
    y   = gsl_vector_alloc(n);      // y-data
    w   = gsl_vector_alloc(n);      // error of data : sigma , standard deviation
    c   = gsl_vector_alloc(3);      // Number of coeffs which wanted to be determined by Least Square Regression
    cov = gsl_matrix_alloc(3, 3);   // Covariant matrix 

    for(i = 0; i < n; i++){

        int count = fscanf (stdin, "%lg %lg %lg", &xi, &yi, &ei);
        if (count != 3){ fprintf (stderr, "error reading file\n"); exit (-1); }

        printf ("%g %g +/- %g\n", xi, yi, ei);
        gsl_matrix_set (X, i, 0, 1.0);
        gsl_matrix_set (X, i, 1, xi);
        gsl_matrix_set (X, i, 2, xi*xi);

        gsl_vector_set (y, i, yi);
        gsl_vector_set (w, i, 1.0/(ei*ei));
        //gsl_vector_set (w, i, 1.0);
    }

    gsl_multifit_linear_workspace * work = gsl_multifit_linear_alloc (n, 3);
    gsl_multifit_wlinear (X, w, y, c, cov, &chisq, work);
    gsl_multifit_linear_free (work);

#define C(i) (gsl_vector_get(c,(i)))
#define COV(i,j) (gsl_matrix_get(cov,(i),(j)))

    printf ("# best fit: Y = %g + %g X + %g X^2\n",C(0), C(1), C(2));

    printf ("# covariance matrix:\n");
    printf ("[ %+.5e, %+.5e, %+.5e  \n",COV(0,0), COV(0,1), COV(0,2));
    printf ("  %+.5e, %+.5e, %+.5e  \n",COV(1,0), COV(1,1), COV(1,2));
    printf ("  %+.5e, %+.5e, %+.5e ]\n",COV(2,0), COV(2,1), COV(2,2));
    printf ("# chisq = %g\n", chisq);

    FILE *fp = fopen("result.dat","w");
    for(int i = 0; i < n; i++){
        double xi = gsl_matrix_get( X, i, 1 );
        double yi = gsl_vector_get( y, i    );
        double wi = gsl_vector_get( w, i    );
        double Y  = C(0) + C(1)*xi + C(2)*xi*xi;
        fprintf(fp, "%e %e %e %e\n", xi, yi, sqrt(1.0/wi), Y );
    }
    
    gsl_matrix_free (X);
    gsl_vector_free (y);
    gsl_vector_free (w);
    gsl_vector_free (c);
    gsl_matrix_free (cov);

    
    return 0;
}
