//
//   y = c0 + c1 * x + c2 * x**2
//
#include <stdio.h>
#include <gsl/gsl_multifit.h>
#include "IOP.h"

int main (int argc, char **argv)
{
    IOParameterSet params(argv[1]);
    int    N       = params.int_value("N_lattice");   // Number of data
    double ln_Dr10 = log( params.value("Dr") );
    if( argc != 3 ){ fprintf (stderr,"usage: a.out param.iop datafile \n"); exit (-1); }

    double* X  = new double [N+1];
    double* V  = new double [N+1];
    double* Vd = new double [N+1];
    double* Vn = new double [N+1];

    FILE *fp = fopen(argv[2], "r");

    double tmp;
    for(int i = 0; i <= N; i++){
        int count = fscanf (fp, "%lf %lf %lf %lf %lf\n", &X[i], &V[i], &tmp, &tmp, &tmp );
        if (count != 5){ fprintf (stderr, "error reading file\n"); exit (-1); }
        Vn[i] = exp( X[i]*ln_Dr10 );
        Vd[i] = V[i] - Vn[i];
    }


    gsl_matrix* x   = gsl_matrix_alloc(N+1, 3);  // X = (1, x, x^2 )
    gsl_vector* y   = gsl_vector_alloc(N+1);     // y-data
    gsl_vector* w   = gsl_vector_alloc(N+1);     // error of data : sigma , standard deviation
    gsl_vector* c   = gsl_vector_alloc(3);       // Number of coeffs determined by this Least Square Regression
    gsl_matrix* cov = gsl_matrix_alloc(3, 3);    // Covariant matrix 

    double xi, yi;
    for(int i = 0; i <= N; i++){

        //int count = fscanf (fp, "%lg %lg %lg", &xi, &yi, &ei);
        //if (count != 3){ fprintf (stderr, "error reading file\n"); exit (-1); }
        // y = x*(1-x)*( C0 + C1*x + C2*x^2 )
        //   = C0*x*(1-x) + C1*x^2*(1-x) + C2*x^3*(1-x) )
        //printf ("%g %g +/- %g\n", xi, yi, ei);

        double x0 = X[i]*(1-X[i]);
        double x1 = X[i]*x0;
        double x2 = X[i]*x1;
        gsl_matrix_set (x, i, 0, x0 );
        gsl_matrix_set (x, i, 1, x1 );
        gsl_matrix_set (x, i, 2, x2 );

        gsl_vector_set (y, i, Vd[i] );
        gsl_vector_set (w, i, 1.0   );
    }

    double chisq;
    gsl_multifit_linear_workspace * work = gsl_multifit_linear_alloc (N+1, 3);
    gsl_multifit_wlinear (x, w, y, c, cov, &chisq, work);
    gsl_multifit_linear_free (work);

#define C(i) (gsl_vector_get(c,(i)))
#define COV(i,j) (gsl_matrix_get(cov,(i),(j)))

    printf ("# best fit: Y = %g + %g X + %g X^2\n",C(0), C(1), C(2));

    printf ("# covariance matrix:\n");
    printf ("[ %+.5e, %+.5e, %+.5e  \n",COV(0,0), COV(0,1), COV(0,2));
    printf ("  %+.5e, %+.5e, %+.5e  \n",COV(1,0), COV(1,1), COV(1,2));
    printf ("  %+.5e, %+.5e, %+.5e ]\n",COV(2,0), COV(2,1), COV(2,2));
    printf ("# chisq = %g\n", chisq);

    FILE *fp2 = fopen("result.dat","w");
    for(int i = 0; i <= N; i++){
        double xi = gsl_matrix_get( x, i, 1 );
        double yi = gsl_vector_get( y, i    );
        double wi = gsl_vector_get( w, i    );
        double x0 = X[i]*(1-X[i]);
        double x1 = X[i]*x0;
        double x2 = X[i]*x1;
        double Y  = C(0)*x0 + C(1)*x1 + C(2)*x2;
        fprintf(fp2, "%e %e %e %e %e %e\n", X[i], yi, sqrt(1.0/wi), Y,  V[i], Vn[i]+Y );
    }

    gsl_matrix_free (x);
    gsl_vector_free (y);
    gsl_vector_free (w);
    gsl_vector_free (c);
    gsl_matrix_free (cov);
    
    return(0);
}
